@@ -99,11 +99,15 @@ struct ovl_data
   int squash_to_root;
   int squash_to_uid;
   int squash_to_gid;
+  int static_nlink;
 
   /* current uid/gid*/
   uid_t uid;
   uid_t gid;
 
+  /* process euid. */
+  uid_t euid;
+
   struct ovl_plugin_context *plugins_ctx;
 };
 
