@@ -70,6 +70,9 @@ umount merged
 
 touch lower/file-lower-layer
 
+# set a "big" xattr
+setfattr -n user.big-xattr -v "$(seq 1000 | tr -d '\n')" lower/file-lower-layer
+
 # no upper layer
 fuse-overlayfs -o lowerdir=lower merged
 
@@ -201,8 +204,52 @@ fi
 touch merged/$(printf %${merged_max_filename_len}s | tr ' ' A})
 
 # If a file is removed but referenced, we must still be able to access it.
-echo 12345 > merged/toremove
-sleep 30 < merged/toremove &
+echo 12345 | tee merged/toremove
+exec 3<> merged/toremove
+sleep 90 &
+exec 3>&-
 sleep_pid=$!
 rm merged/toremove
-grep 12345 /proc/$sleep_pid/fd/0
+grep 12345 /proc/$sleep_pid/fd/3
+
+touch merged/a merged/b
+chmod 6 merged/a
+mv merged/a merged/x
+mv merged/b merged/a
+
+# https://github.com/containers/fuse-overlayfs/issues/279
+umount -l merged
+
+rm -rf lower upper workdir merged
+mkdir lower upper workdir merged
+mkdir lower/test
+touch lower/test/a.txt
+
+fuse-overlayfs -o lowerdir=lower,upperdir=upper,workdir=workdir merged
+
+(cd merged/test; touch a.txt; mv a.txt a2.txt; touch a3.txt; ln -s a3.txt a.txt)
+
+if test -e upperdir/test/.wh.a.txt; then
+   echo "whiteout file still exists" >&2
+   exit 1
+fi
+
+# https://github.com/containers/fuse-overlayfs/issues/306
+umount -l merged
+
+rm -rf lower upper workdir merged
+mkdir lower upper workdir merged
+
+mkdir -p lower/a/b
+fuse-overlayfs -o lowerdir=lower,upperdir=upper,workdir=workdir merged
+
+rm -rf merged/a
+mkdir -p merged/a/b
+rm -rf merged/a/b
+test \! -e upper/a/b
+
+mknod merged/dev-foo c 10 175
+attr -l merged/dev-foo
+
+umount merged
+
